<?php

class SmartPayGateway extends PaymentModule
{
    private $_html = '';
    private $_postErrors = array();
    
    public function __construct()
    {
        $this->name        = 'smartpaygateway';
        $this->displayName = 'SmartCash Payment Gateway';
        $this->tab         = 'payments_gateways';
        $this->version     = 0.1;
        $this->author = 'Stonewall Capital LTD';
        $this->bootstrap = true;
        
        $config = Configuration::getMultiple(array(
            'SMART_PAY_KEY',
            'SMART_PAY_SECRET'
        ));
        
        if (isset($config['SMART_PAY_KEY']))
            $this->SC_KEY = $config['SMART_PAY_KEY'];
        if (isset($config['SMART_PAY_SECRET']))
            $this->SC_SECRET = $config['SMART_PAY_SECRET'];
        
        
        parent::__construct();
        
        /* The parent construct is required for translations */
        $this->page        = basename(__FILE__, '.php');
        $this->description = $this->l('Accept payments with SmartCash');
        
        if (!isset($this->SMART_PAY_KEY) OR !isset($this->SMART_PAY_SECRET))
            $this->warning = $this->l('your SmartCash API Credentials must be configured in order to use this module correctly');
        
        
    }
    
    
    function install()
    {
        //Call PaymentModule default install function
        $install = parent::install() && $this->registerHook('payment') && $this->registerHook('paymentReturn') && $this->registerHook('displayMobileHeader');
        
        
        Configuration::updateValue('SMART_PAY_KEY', "");
        Configuration::updateValue('SMART_PAY_SECRET', "");
        return $install;
    }
    
    
    function uninstall()
    {
        Configuration::deleteByName('SMART_PAY_KEY');
        Configuration::deleteByName('SMART_PAY_SECRET');
        return parent::uninstall();
    }
    
    
    function getContent()
    {
        $this->_html = '<h2>' . $this->displayName . '</h2>';
        
        if (!empty($_POST)) {
            $this->_postValidation();
            if (!sizeof($this->_postErrors))
                $this->_postProcess();
            else
                foreach ($this->_postErrors as $err)
                    $this->_html .= "<div class='alert error'>{$err}</div>";
        } else {
            $this->_html .= "<br />";
        }
        
        $this->_html .= $this->_displaycheckout();
        $this->_displayForm();
        
        return $this->_html;
    }
    
    function checkTotal($cart)
    {
        global $cookie, $smarty;
        
        $check_total  = 0;
        $cart_details = $cart->getSummaryDetails(null, true);
        //products
        $products     = $cart->getProducts();
        foreach ($products as $product) {
            $check_total += $product['price'] * $product['quantity'];
        }
        //shipping
        if (_PS_VERSION_ < '1.5') {
            $shipping = $cart_details['total_shipping_tax_exc'];
        } else {
            $shipping = $this->context->cart->getTotalShippingCost();
        }
        $check_total += $shipping;
        $check_total += $cart_details['total_tax'];
        $check_total -= $cart_details['total_discounts_tax_exc'];
        return $check_total;
    }
    
    function execPayment($cart)
    {
       
        
        global $cookie, $smarty;
        
        //Verify currencies and display payment form
        $cart_details          = $cart->getSummaryDetails(null, true);
        $currencies            = Currency::getCurrencies();
        $authorized_currencies = array_flip(explode(',', $this->currencies));
        $currencies_used       = array();
        foreach ($currencies as $key => $currency)
            if (isset($authorized_currencies[$currency['id_currency']]))
                $currencies_used[] = $currencies[$key];
        
        $order_currency = '';
        
        foreach ($currencies_used as $key => $currency) {
            if ($currency['id_currency'] == $cart->id_currency) {
                $order_currency = $currency['iso_code'];
            }
        }
        
        $products = $cart->getProducts();
        foreach ($products as $key => $product) {
            $products[$key]['name'] = str_replace('"', '\'', $product['name']);
            $products[$key]['name'] = htmlentities(utf8_decode($product['name']));
        }
        
        $discounts = $cart_details['discounts'];
        
        $carrier = $cart_details['carrier'];
        
        if (_PS_VERSION_ < '1.5') {
            $shipping_cost = $cart_details['total_shipping_tax_exc'];
        } else {
            $shipping_cost = $this->context->cart->getTotalShippingCost();
        }
        
        $CheckoutUrl   = 'https://www.smart-cash.it/smartPay/initializePayment.php';
        $cart_order_id = $cart->id;
      
  
        
        
        $total             = number_format($cart->getOrderTotal(true, 3), 2, '.', '');
        
        $manufacturers =array();// Manufacturer::getManufacturers($this->context->language->id);
        $employees = Employee::getEmployees($this->context->language->id);


        foreach($employees as $employee){

            if(isset($employee['id_employee'])){
            $temp = new Employee($employee['id_employee']);

            if(isset($temp->id)){
                if(isset($temp->id_manufacturer) && $temp->id_manufacturer != 0){
                   $manufacturers[$temp->id_manufacturer] = $temp;

                }
            }
        }
        }

       

        
        $params                         = array();
        $params['merchantUsername'] = Configuration::get('SMART_PAY_KEY');
        $params['merchantPassword'] = Configuration::get('SMART_PAY_SECRET');
        $params['paymentMethod'] = "SC_PAY_USING_CBK";

        $params['redirectTo']           = base64_encode($this->context->link->getModuleLink('smartpaygateway', 'validation', array(), true));
        $params['jsonCart']['cart_id']    = $cart_order_id;
        $params['jsonCart']['total'] = $total;
        $params['jsonCart']['currency'] = $order_currency;

         if ($shipping_cost > 0) {
                $params['jsonCart']['shipping'] = $shipping_cost;
            }
            if ($cart_details['total_tax'] > 0) {
                $params['jsonCart']['additional'] = $cart_details['total_tax'];
            }
            if ($cart_details['total_discounts_tax_exc'] > 0) {
               $params['jsonCart']['discount'] =  $cart_details['total_discounts_tax_exc'];
            }
           






            $i              = 0;
            $imageHandle = new Image();
            $linkHandle = new Link();
            foreach ($products as $product) {
                
                $image = $imageHandle->getCover($product['id_product']);
                $imagePath = $linkHandle->getImageLink($product['link_rewrite'], $image['id_image'], 'home_default');
                $params['jsonCart']['values'][$i]['title']          = trim(preg_replace('/\s+/', ' ', (htmlentities(strip_tags($product['name']),ENT_QUOTES)))); 
                $params['jsonCart']['values'][$i]['description'] = trim(preg_replace('/\s+/', ' ', (htmlentities(strip_tags($product['description_short']),ENT_QUOTES)))); 
                $params['jsonCart']['values'][$i]['quantity']    = $product['quantity'];
                $params['jsonCart']['values'][$i]['image']          = $imagePath;
                $params['jsonCart']['values'][$i]['amount']       = $product['price'];
                $params['jsonCart']['values'][$i]['shipping']       = 0;
                $params['jsonCart']['values'][$i]['additional']       = 0;
                if(isset($product['id_manufacturer']) && isset($manufacturers[$product['id_manufacturer']])){
                    $params['jsonCart']['values'][$i]['employee'] = $manufacturers[$product['id_manufacturer']]->smart_cash_vendor_token;
                }
                $i++;
                unset($image);
                unset($imagePath);
            }
           
            
            
            $params['jsonCart'] = base64_encode(json_encode($params['jsonCart']));

        $authUrl = $CheckoutUrl . '?' . http_build_query($params);
        $result = file_get_contents($authUrl);
        $result = json_decode($result,1);

        if(!$result){
            die("<div style='border: 1px solid;margin: 10px 0px;padding:15px 10px 15px 50px;background-repeat: no-repeat;background-position: 10px center;color: #D8000C;background-color: #FFBABA;'>Errore del Server : JSON non valido.<a href = '".$this->context->link->getPageLink('order', true, NULL, "step=3")."'>Torna indietro</a></div>");
        }


        if($result['response']['transaction_status'] == 0){
             die("<div style='border: 1px solid;margin: 10px 0px;padding:15px 10px 15px 50px;background-repeat: no-repeat;background-position: 10px center;color: #D8000C;background-color: #FFBABA;'>Errore : ".$result['response']['transaction_message'].".<a href = '".$this->context->link->getPageLink('order', true, NULL, "step=3")."'>Torna indietro</a></div>");
        }else{
            $redirectUrl = $result['response']['transaction_action_url'];
            header('Location: ' . $redirectUrl);

        }


        
    }
    
    
    function hookPayment($params)
    {
        global $smarty;
        $smarty->assign(array(
            'this_path' => $this->_path,
            'this_path_ssl' => Configuration::get('PS_FO_PROTOCOL') . $_SERVER['HTTP_HOST'] . __PS_BASE_URI__ . "modules/{$this->name}/"
        ));
        
        return $this->display(__FILE__, 'payment.tpl');
    }
    
    
    function hookPaymentReturn($params)
    {
        global $smarty;
        $state = $params['objOrder']->getCurrentState();
        if ($state == _PS_OS_OUTOFSTOCK_ or $state == _PS_OS_PAYMENT_)
            $smarty->assign(array(
                'total_to_pay' => Tools::displayPrice($params['total_to_pay'], $params['currencyObj'], false, false),
                'status' => 'ok',
                'id_order' => $params['objOrder']->id
            ));
        else
            $smarty->assign('status', 'failed');
        
        return $this->display(__FILE__, 'payment_return.tpl');
    }
    
    
    private function _postValidation()
    {
        if (isset($_POST['btnSubmit'])) {
            if (empty($_POST['KEY']))
                $this->_postErrors[] = $this->l('Your SmartCash API KEY is required.');
            if (empty($_POST['SECRET']))
                $this->_postErrors[] = $this->l('Your SmartCash API SECRET is required.');
        }
        
    }
    
    
    private function _postProcess()
    {
        if (isset($_POST['btnSubmit'])) {
            Configuration::updateValue('SMART_PAY_KEY', $_POST['KEY']);
            Configuration::updateValue('SMART_PAY_SECRET', $_POST['SECRET']);
        }
        
        $ok      = $this->l('Ok');
        $updated = $this->l('Settings Updated');
        return $this->_html .= "<div class='conf confirm'><img src='../img/admin/ok.gif' alt='{$ok}' />{$updated}</div>";
    }
    
    
    private function _displaycheckout()
    {
        $modDesc    = $this->l('This module allows you to accept payments using Smart Cash.',"smartpaygateway");
        $modStatus  = $this->l('Smart Cash online payment service could be the right solution for you',"smartpaygateway");
        $modconfirm = $this->l('');
        $this->_html .= "<img src='../modules/smartpaygateway/smartCashLogo.png' style='float:left; margin-right:15px;width:100px' />
                                        <b>{$modDesc}</b>
                                        <br />
                                        <br />
                                        {$modStatus}
                                        <br />
                                        {$modconfirm}
                                        <br />
                                        <br />
                                        <br />";
    }
    
    
    private function _displayForm()
    {
        $modcheckout          = $this->l('SmartCash Setup',"smartpaygateway");
        $modcheckoutDesc      = $this->l('Please specify the SmartCash API KEY and API SECRET.',"smartpaygateway");
        $modClientLabelSid    = $this->l('SmartCash API KEY',"smartpaygateway");
        $modClientValueSid    = Configuration::get('SMART_PAY_KEY');
        $modClientLabelSecret = $this->l('SmartCash API SECRET');
        $modClientValueSecret = Configuration::get('SMART_PAY_SECRET');
        $modUpdateSettings    = $this->l('Update settings');
        $this->_html .= "
        <br />
        <br />
        <p><form action='{$_SERVER['REQUEST_URI']}' method='post'>
                <fieldset>
                <legend><img src='../img/admin/access.png' />{$modcheckout}</legend>
                        <table border='0' width='500' cellpadding='0' cellspacing='0' id='form'>
                                <tr>
                                        <td colspan='2'>
                                                {$modcheckoutDesc}<br /><br />
                                        </td>
                                </tr>
                                <tr>
                                        <td width='130'>{$modClientLabelSid}</td>
                                        <td>
                                                <input type='text' name='KEY' value='{$modClientValueSid}' style='width: 300px;' />
                                        </td>
                                </tr>
                                <tr>
                                        <td width='130'>{$modClientLabelSecret}</td>
                                        <td>
                                                <input type='text' name='SECRET' value='{$modClientValueSecret}' style='width: 300px;' />
                                        </td>
                                </tr>
                                
                                <tr>
                                        <td colspan='2' align='center'>
                                                <input class='button' name='btnSubmit' value='{$modUpdateSettings}' type='submit' />
                                        </td>
                                </tr>
                        </table>
                </fieldset>
        </form>
        </p>
        <br />
        <br />
       ";
    }
}
